<?php
/**
 * SchwarzesBrettPlugin.class.php
 *
 * Plugin zum Verwalten von Schwarzen Brettern (Angebote und Gesuche)
 *
 * Diese Datei enthlt die Hauptklasse des Plugins
 *
 * @author		Jan Kulmann <jankul@zmml.uni-bremen.de>
 * @author		Michael Riehemann <michael.riehemann@uni-oldenburg.de>
 * @package 	IBIT_SchwarzesBrettPlugin
 * @copyright 	2008-2010 IBIT und ZMML
 * @license 	http://www.gnu.org/licenses/gpl.html GPL Licence 3
 * @version 	2.0a
 */

// IMPORTS
require_once 'bootstrap.inc.php';

/**
 * SchwarzesBrettPlugin Hauptklasse
 *
 */
class SchwarzesBrettPlugin extends StudIPPlugin implements SystemPlugin
{
	public $zeit, $announcements, $user, $perm;
	private $template_factory, $layout, $layout_infobox;

	const THEMEN_CACHE_KEY = 'plugins/SchwarzesBrettPlugin/themen';
	const ARTIKEL_CACHE_KEY = 'plugins/SchwarzesBrettPlugin/artikel/';

    /**
     *
     */
    public function __construct()
    {
        global $user, $perm, $auth;
        parent::__construct();

        $this->user = $user;
        $this->perm = $perm;
        $this->template_factory = new Flexi_TemplateFactory(dirname(__FILE__).'/templates');
        $this->layout =  $GLOBALS['template_factory']->open('layouts/base_without_infobox');
        $this->layout_infobox =  $GLOBALS['template_factory']->open('layouts/base');

        // Holt die Laufzeit aus der Config. Default: 30Tage
        $this->zeit = get_config('BULLETIN_BOARD_DURATION') * 24 * 60 * 60;
        // Holt Anzahl anzuzeigende neuste Anzeigen. Default: 20
        $this->announcements = (int)get_config('BULLETIN_BOARD_ANNOUNCEMENTS');

        //Navigation
        $nav = new AutoNavigation(_('Schwarzes Brett'), PluginEngine::getURL($this, array()));
        $nav->setImage($this->getPluginIcon());
        Navigation::addItem('/schwarzesbrettplugin', $nav);

        // Sachen in den Header laden (bis 1.11)
        $GLOBALS['_include_additional_header'] .= '<script src="'.$this->getPluginURL().'/js/schwarzesbrett.js" type="text/javascript"></script>'."\n";
    }

    /**
     * @return Grafik, die in der Hauptnavigation angezeigt wird
     */
    private function getPluginIcon()
    {
        $last_visitdate = DBManager::get()->query("SELECT MAX(last_visitdate) FROM sb_visits WHERE user_id='{$this->user->id}'")->fetchColumn();
        $last_artikel = DBManager::get()->query("SELECT count(*) FROM sb_artikel WHERE mkdate > '{$last_visitdate}' AND visible = 1")->fetchColumn();
        if ($last_artikel > 0) {
            if ($GLOBALS['STUDIP_INSTALLATION_ID'] == "uol" || $GLOBALS['STUDIP_INSTALLATION_ID'] == "uni-ol") {
                $image = '/images/paste_plain2.png';
            } else {
                $image = '/images/header_pinn2.gif';
            }
        } else {
            if ($GLOBALS['STUDIP_INSTALLATION_ID'] == "uol" || $GLOBALS['STUDIP_INSTALLATION_ID'] == "uni-ol") {
                $image = '/images/paste_plain.png';
            } else {
                $image = '/images/header_pinn1.gif';
            }
        }
        return $this->getPluginURL().$image;
    }

    /**
     * Hauptfunktion, dient in diesem Plugin als Frontcontroller und steuert die Ausgaben
     *
     */
    public function show_action()
    {
        //Suchergebnisse abfragen und anzeigen, falls vorhanden
        if (Request::get('modus') == "show_search_results") {
            $this->search(Request::get('search_text'));
            return;
        }
        $this->showThemen();
    }


    /**
     * Zeigt die Seite zum Erstellen oder Bearbeiten von Artikeln
     */
    public function editArtikel_action()
    {
        //Daten holen
        $a = new Artikel(Request::get('artikel_id'));
        if (!Request::get('artikel_id')) {
            $a->setThemaId(Request::get('thema_id'));
        }

        //Speichern
        if (Request::get('modus') == "add_artikel" && $this->perm->have_perm($this->getThemaPermission(Request::get('thema_id')))) {
            if ((!$this->isDuplicate(Request::get('titel')) && Request::get('artikel_id')) || !Request::get('artikel_id')) {
                if (Request::get('titel') && Request::get('beschreibung')) {
                    $a = new Artikel(Request::get('artikel_id'));
                    $a->setTitel(Request::get('titel'));
                    $a->setBeschreibung(Request::get('beschreibung'));
                    $a->setThemaId(Request::get('thema_id'));
                    $a->setVisible(Request::get('visible', 0));
                    $a->save();
                    $this->message =  MessageBox::success("Die Anzeige wurde erfolgreich gespeichert.");
                    //nach dem verndern der themen, muss auch der cache geleert werden
                    StudipCacheFactory::getCache()->expire(self::ARTIKEL_CACHE_KEY.$a->getThemaId());
                    StudipCacheFactory::getCache()->expire(self::THEMEN_CACHE_KEY);
                    $this->show_action();
                    return;
                } else {
                    $this->message = MessageBox::error("Bitte geben Sie einen Titel und eine Beschreibung an.");
                }
            } elseif($this->isDuplicate(Request::get('titel'))) {
                $this->message = MessageBox::error("Sie haben bereits einen Artikel mit diesem Titel erstellt. Bitte beachten Sie die Nutzungshinweise!");
            } else {
                $this->message = MessageBox::error("Sie haben nicht die erforderlichen Rechte eine Anzeige zu erstellen.");
            }
        }

        //Ausgabe
        $template = $this->template_factory->open('edit_artikel');
        $template->message = $this->message;
        $template->set_layout($this->layout_infobox);
        $template->set_attribute('thema_id', Request::get('thema_id'));
        $template->set_attribute('themen', $this->getThemen());
        $template->set_attribute('a', $a);
        $template->set_attribute('zeit', $this->zeit);
        $template->set_attribute('link', PluginEngine::getURL($this, array(), 'show'));
        $template->set_attribute('link_thema', PluginEngine::getURL($this, array("open" => Request::get('thema_id')), 'editArtikel'));

        //Infobox
        $template->infobox = array(
            'picture' => 'contract.jpg',
            'content' => array(array(
                "kategorie" => _("Information:"),
                "eintrag" => array(
                    array("icon" => $this->getPluginURL(). '/images/information.png',
                    "text" => 'Jede Anzeige <b>sollte</b> einen <b>universitren Bezug</b> haben, alle anderen Anzeigen werden entfernt.'),
                    array("icon" => $this->getPluginURL(). '/images/information.png',
                    "text" => '<b>Bitte Artikel nur in <em>eine</em> Kategorie einstellen!</b>'),
                    array("icon" => $this->getPluginURL(). '/images/information.png',
                    "text" => 'Sobald eine Anzeige nicht mehr aktuell ist (z.b. in dem Fall, dass ein Buch verkauft oder eine Mitfahrgelegenheit gefunden wurde), sollte die Anzeige durch den Autor entfernt werden.'),
                    array("icon" => $this->getPluginURL(). '/images/information.png',
                    "text" => 'Unter der Beschreibung wird automatisch ein Link zu Ihrer Benutzerhomepage eingebunden. <br />Auerdem knnen andere Nutzer direkt ber einen Button antworten. Diese Nachrichten erhalten Sie als Stud.IP interne Post!'),
                    #array("icon" => $pluginpfad. '/images/information.png',
                    #"text" => 'Bitte die Anzeigen in die dafr vorgesehenen Themen einstellen, damit dieses schwarze Brett so bersichtlich wie mglich bleibt.'),
                    array("icon" => $this->getPluginURL(). '/images/information.png',
                    "text" => 'Wird ein Gegenstand oder eine Dienstleistung gegen Bezahlung angeboten, sollte der Betrag genannt werden, um unntige Nachfragen zu vermeiden.'),
                    array("icon" => $this->getPluginURL(). '/images/information.png',
                    "text" => 'Jede Anzeige, die gegen diese Nutzungsordnung verstt, wird umgehend entfernt.')
                )
            ))
        );
        echo $template->render();
    }

    /**
     * Zeigt das Formular zum Erstellen oder Bearbeiten von Themen an.
     * Nur fr root
     */
    public function editThema_action()
    {
        if ($this->perm->have_perm('root')) {
            $t = new Thema(Request::get('thema_id'));

            // Speichern
            if (Request::get('modus') == "save_thema") {
                if (Request::get('titel')) {
                    $t->setTitel(Request::get('titel'));
                    $t->setBeschreibung(Request::get('beschreibung'));
                    $t->setPerm(Request::get('thema_perm'));
                    $t->setVisible(Request::get('visible', 0));
                    $t->save();

                    $this->message = MessageBox::success("Das Thema wurde erfolgreich gespeichert.");
                    //nach dem verndern der themen, muss auch der cache geleert werden
                    StudipCacheFactory::getCache()->expire(self::THEMEN_CACHE_KEY);
                    $this->showThemen();
                    return;
                } else {
                    $this->message = MessageBox::error("Bitte geben Sie einen Titel (Pflichtfeld) ein.");
                }
            }

            // Ausgabe
            $template = $this->template_factory->open('edit_thema');
            $template->set_layout($this->layout);
            $template->message = $this->message;
            $template->set_attribute('t', $t);
            $template->set_attribute('link', PluginEngine::getURL($this, array(), 'editThema'));
            $template->set_attribute('link_exit', PluginEngine::getURL($this, array(), 'show'));
            echo $template->render();
        }
    }

    /**
     * Man kann mit dieser Funktion alle veralteten Artikel aus der DB lschen
     * Nur fr Root
     *
     */
    public function deleteOldArtikel_action()
    {
        if ($this->perm->have_perm('root')) {
            $artikel = DBManager::get()->query("SELECT artikel_id FROM sb_artikel WHERE UNIX_TIMESTAMP() > (mkdate + {$this->zeit})")->fetchAll(PDO::FETCH_COLUMN);
            foreach ($artikel as $id) {
                $a = new Artikel($id);
                $a->delete();
            }
            if (count($artikel) > 0) {
                $this->message = MessageBox::success("Es wurden erfolgreich <em>".count($artikel)."</em> Artikel aus der Datenbank gelscht.");
            } else {
                $this->message = MessageBox::info("Es gibt keine Artikel in der Datenbank, die gelscht werden knnen.");
            }
        }
        $this->showThemen();
    }

    /**
     * Lscht ein Thema mit all seinen Anzeigen inkl. Sicherheitsabfrage
     * Nur fr Root
     */
    public function deleteThema_action()
    {
        if ($this->perm->have_perm('root')) {
            //Thema lschen Sicherheitsabfrage
            if (Request::get('modus') == "delete_thema_really") {
                $t = new Thema(Request::get('thema_id'));
                $t->delete();
                $this->message =  MessageBox::success("Das Thema und alle dazugehrigen Anzeigen wurden erfolgreich gelscht.");
                //nach dem verndern der themen, muss auch der cache geleert werden
                StudipCacheFactory::getCache()->expire(self::THEMEN_CACHE_KEY);
            } else {
                $t = new Thema($_REQUEST['thema_id']);
                echo $this->createQuestion('Soll das Thema **'.$t->getTitel().'** wirklich gelscht werden?', array("modus"=>"delete_thema_really", "thema_id"=>$t->getThemaId()), 'deleteThema');
            }
        }
        $this->showThemen();
    }

    /**
     *  Lscht eine Anzeige inkl. Sicherheitsabfrage
     */
    public function deleteArtikel_action()
    {
        $a = new Artikel(Request::get('artikel_id'));

        //Artikel lschen Sicherheitsabfrage
        if (Request::get('modus') == "delete_artikel_really") {
            //Root lscht Artikel eines Benutzers, also diesen benachrichtigen.
            if ($a->getUserId() != $this->user->id && $this->perm->have_perm('root')) {
                $messaging = new messaging();
                $msg = sprintf("Die Anzeige \"%s\" wurde von der Administration gelscht.", $a->getTitel());
                $messaging->insert_message($msg, get_username($a->getUserId()), "____%system%____", FALSE, FALSE, 1, FALSE, "Schwarzes Brett: Anzeige gelscht!");
            }
            $a->delete();
            $this->message = MessageBox::success("Die Anzeige wurde erfolgreich gelscht.");
            //nach dem verndern der themen, muss auch der cache geleert werden
            StudipCacheFactory::getCache()->expire(self::ARTIKEL_CACHE_KEY.$a->getThemaId());
            StudipCacheFactory::getCache()->expire(self::THEMEN_CACHE_KEY);
        } elseif ($a->getUserId() == $this->user->id || $this->perm->have_perm('root')) {
            echo $this->createQuestion('Soll die Anzeige **'.$a->getTitel().'** von %%'.get_fullname($a->getUserId()).'%% wirklich gelscht werden?', array("modus"=>"delete_artikel_really", "artikel_id"=>$a->getArtikelId()), 'deleteArtikel');
        } else {
            $this->message = MessageBox::error("Sie haben nicht die Berechtigung diese anzeige zu lschen.");
        }
        $this->showThemen();
    }

    /**
     * Gibt alle Anzeigen zu einem Thema zurck
     *
     * @uses StudipCache
     *
     * @param string $thema_id
     * @return array Anzeigen
     */
    private function getArtikel($thema_id)
    {
        $cache = StudipCacheFactory::getCache();
        $ret = unserialize($cache->read(self::ARTIKEL_CACHE_KEY.$thema_id));

        if (empty($ret)) {
            $ret = array();
            $artikel_ids = DBManager::get()->query("SELECT artikel_id FROM sb_artikel "
                ."WHERE thema_id='{$thema_id}' AND UNIX_TIMESTAMP() < (mkdate + {$this->zeit}) "
                ."AND (visible=1 OR (visible=0 AND (user_id='{$this->user->id}' "
                ."OR 'root'='{$this->perm->get_perm($this->user->id)}'))) "
                ."ORDER BY mkdate DESC")->fetchAll(PDO::FETCH_COLUMN);
            foreach ($artikel_ids as $artikel_id) {
                $ret[] = new Artikel($artikel_id);
            }
            $cache->write(self::ARTIKEL_CACHE_KEY.$thema_id, serialize($ret));
        }
        return $ret;
    }

    /**
     * Gibt die Anzahl Anzeigen fr ein Thema zurck
     *
     * @param md5 $thema_id
     * @return int
     */
    private function getArtikelCount($thema_id)
    {
        return DBManager::get()->query("SELECT count(*) FROM sb_artikel "
            ."WHERE thema_id='{$thema_id}' AND UNIX_TIMESTAMP() < (mkdate + {$this->zeit}) "
            ."AND (visible=1 OR (visible=0 AND (user_id='{$this->user->id}' "
            ."OR 'root'='{$this->perm->get_perm($this->user-userid)}'))) ")->fetchColumn();
    }

    /**
     * Gibt die Anzahl Besucher eines Artikels zurck.
     *
     * @param string $artikel_id
     * @return int Anzahl Besucher
     */
    private function getArtikelLookups($artikel_id)
    {
        return DBManager::get()->query("SELECT COUNT(*) FROM sb_visits WHERE type='artikel' AND object_id='{$artikel_id}'")->fetch(PDO::FETCH_COLUMN);
    }

    /**
     * Gibt eine Liste aller Themen aus der Datenbank zurck, die sichtbar sind
     * oder in denen der Benutzer bereits einen Artikel erstellt hat.
     *
     * @uses StudipCache
     *
     * @return array Liste aller Themen
     */
    private function getThemen()
    {
        $cache = StudipCacheFactory::getCache();
        $ret = unserialize($cache->read(self::THEMEN_CACHE_KEY));

        if(empty($ret)) {
            $themen = DBManager::get()->query("SELECT t.thema_id, COUNT(a.thema_id) count_artikel "
                    ."FROM sb_themen t LEFT JOIN sb_artikel a USING (thema_id) "
                    ."WHERE t.visible=1 OR t.user_id='{$this->user->id}' "
                    ."OR 'perm'='{$this->perm->get_perm($this->user->id)}' "
                    ."GROUP BY t.thema_id ORDER BY t.titel")->fetchAll(PDO::FETCH_ASSOC);
            $ret = array();
            foreach ($themen as $thema) {
                $t = new Thema($thema['thema_id']);
                $t->setArtikelCount($this->getArtikelCount($thema['thema_id']));
                array_push($ret, $t);
            }
            $cache->write(self::THEMEN_CACHE_KEY, serialize($ret));
        }
        return $ret;
    }

    /**
     * Gibt die Benutzerrechte eines Themas zurck
     *
     * @param string $thema_id
     * @return string $permission
     */
    private function getThemaPermission($thema_id)
    {
        return DBManager::get()->query("SELECT perm FROM sb_themen WHERE thema_id='{$thema_id}'")->fetch(PDO::FETCH_COLUMN);
    }

    /**
     * berprft, ob eine Anzeige bereits vorhanden ist, dabei werden
     * Titel, UserID und Datum verglichen.
     *
     * @param string $titel
     * @return boolean
     */
    private function isDuplicate($titel)
    {
        if (count(DBManager::get()->query("SELECT artikel_id FROM sb_artikel "
              ."WHERE user_id='{$this->user->id}' AND titel='{$titel}' "
              ."AND mkdate>(UNIX_TIMESTAMP()-(60*60*24))")->fetchAll(PDO::FETCH_ASSOC)) > 0) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * berprft, ob der Benutzer dieses Objekt (Thema oder Artikel) bereits angesehen hat.
     *
     * @param string $obj_id
     * @return datetime oder boolean
     */
    private function hasVisited($obj_id)
    {
        $last_visitdate = DBManager::get()->query("SELECT last_visitdate FROM sb_visits WHERE object_id='{$obj_id}' AND user_id='{$this->user->id}'")->fetch(PDO::FETCH_COLUMN);
        if (!empty($last_visitdate)) {
            return $last_visitdate;
        } else {
            return false;
        }
    }

    /**
     * Fhrt die Suche nach Anzeigen durch und zeigt die Ergebnisse an.
     *
     * @param String $search_text Suchwort
     */
    private function search($search_text)
    {
        //Benutzereingaben abfangen (Wrter krzer als 3 Zeichen)
        if(empty($search_text) || strlen($search_text) < 3)
        {
            $this->message = MessageBox::error("Ihr Suchwort ist zu kurz, bitte versuchen Sie es erneut!");
            $this->showThemen();
            return;
        }

        //Datenbankabfrage
        $sql = sprintf("SELECT a.thema_id, a.artikel_id, a.titel, t.titel t_titel FROM sb_artikel AS a, sb_themen AS t WHERE
                t.thema_id=a.thema_id AND (UPPER(a.titel) LIKE '%s' OR UPPER(a.beschreibung) LIKE '%s') AND UNIX_TIMESTAMP() < (a.mkdate + %d)
                AND (a.visible=1 OR (a.visible=0 AND (a.user_id='%s' OR 'root'='%s'))) ORDER BY t.titel, a.titel
            ","%".strtoupper($search_text)."%","%".strtoupper($search_text)."%", $this->zeit, $this->user->id, $this->perm->get_perm($this->user->id));
        $dbresults = DBManager::get()->query($sql)->fetchAll(PDO::FETCH_ASSOC);

        // keine Ergebnisse vorhanden
        if(count($dbresults) == 0) {
            $this->message = MessageBox::error("Es wurden fr <em>{$search_text}</em> keine Ergebnisse gefunden.");
            $this->showThemen();
            return;
        }

        //Ergebnisse anzeigen
        $results = array();
        $thema = array();
        foreach ($dbresults as $result) {
            $a = new Artikel($result['artikel_id']);
            if(empty($thema['thema_id'])) {
                $thema['thema_id'] = $result['thema_id'];
                $thema['thema_titel'] = htmlReady($result['t_titel']);
                $thema['artikel'] = array();

            } elseif($result['thema_id'] != $thema['thema_id']) {
                array_push($results, $thema);
                unset($thema);
                $thema['thema_id'] = $result['thema_id'];
                $thema['thema_titel'] = htmlReady($result['t_titel']);
                $thema['artikel'] = array();
            }
            array_push($thema['artikel'], $this->showArtikel($a));
        }
        array_push($results, $thema);

        //Ausgabe erzeugen
        $template = $this->template_factory->open('search_results');
        $template->set_layout($this->layout);
        $template->set_attribute('zeit', $this->zeit);
        $template->set_attribute('pluginpfad', $this->getPluginURL());
        $template->set_attribute('link_search', PluginEngine::getURL($this, array("modus"=>"show_search_results")));
        $template->set_attribute('link_back', PluginEngine::getURL($this, array(), 'show'));
        $template->set_attribute('results', $results);
        echo $template->render();
    }

    /**
     * Zeigt alle Themen und Anzeigen an
     *
     */
    private function showThemen()
    {
        $themen = $this->getThemen();

        $template = $this->template_factory->open('show_themen');
        $template->set_layout($this->layout);
        $template->message = $this->message;
        $template->set_attribute('zeit', $this->zeit);
        $template->set_attribute('pluginpfad', $this->getPluginURL());
        $template->set_attribute('link_edit', PluginEngine::getURL($this, array(), 'editThema'));
        $template->set_attribute('link_artikel', PluginEngine::getURL($this, array(), 'editArtikel'));
        $template->set_attribute('link_delete', PluginEngine::getURL($this, array(), 'deleteThema'));
        $template->set_attribute('link_search', PluginEngine::getURL($this, array("modus" => "show_search_results")));
        $template->set_attribute('link_back', PluginEngine::getURL($this, array()));
        $template->set_attribute('last_visit_date', $this->last_visitdate);

        //Keine themen vorhanden
        if (count($themen) == 0) {
            $template->set_attribute('keinethemen', TRUE);
        }
        //themen anzeigen
        else {
            //Anzahl Themen pro Spalte berechnen
            if(count($themen) > 6) { //3 Spalten
                $template->set_attribute('themen_rows', (count($themen)%3==0)? count($themen)/3 : (count($themen)/3)+1);
            } elseif(count($themen) > 2) { //2 Spalten
                $template->set_attribute('themen_rows', 2);
            } else { //1 Spalte
                $template->set_attribute('themen_rows', 1);
            }

            $results = array();
            $thema = array();
            foreach ($themen as $tt) {
                $thema['thema'] = $tt;
                if($this->perm->have_perm($tt->getPerm(), $this->user->id) ||  $this->perm->have_perm('root')) {
                    $thema['permission'] = true;
                }
                $thema['artikel'] = array();
                $thema['last_thema_user_date'] = DBManager::get()->query("SELECT MAX(sv.last_visitdate) FROM sb_visits AS sv LEFT JOIN sb_artikel AS sa ON sv.object_id = sa.artikel_id WHERE sv.user_id='{$this->user->id}' AND sa.thema_id = '{$tt->getThemaId()}'")->fetchColumn();
                $thema['countArtikel'] = $tt->getArtikelCount();
                array_push($results, $thema);
            }
            $template->set_attribute('results', $results);

            $newOnes = $this->getLastArtikel();
            if (count($newOnes) > 0) {
                foreach($newOnes as $a) {
                    $lastArtikel[] = $this->showArtikel($a, 'show_lastartikel');
                }
                $template->set_attribute('lastArtikel', $lastArtikel);
            }
        }
        //Adminfunktionen anzeigen
        if ($this->perm->have_perm('root')) {
            $template->set_attribute('old_entries', DBManager::get()->query("SELECT count(artikel_id) FROM sb_artikel WHERE UNIX_TIMESTAMP() > (mkdate + {$this->zeit})")->fetchColumn());
            $template->set_attribute('rootlinknew', PluginEngine::getURL($this, array(), 'editThema'));
            $template->set_attribute('rootlinkdelete', PluginEngine::getURL($this, array(), 'deleteOldArtikel'));
            $template->set_attribute('rootaccess', true);
        }
        echo $template->render();
    }

    /**
     * Zeigt eine Anzeige an (wird per ajax geholt)
     *
     * @param Object $a eine Anzeige
     * @param string $template
     */
    private function showArtikel($a, $template = 'show_artikel')
    {
        $template = $this->template_factory->open($template);
        $template->set_attribute('zeit', $this->zeit);
        $template->set_attribute('a', $a);
        $template->set_attribute('anzahl', $this->getArtikelLookups($a->getArtikelId()));
        $template->set_attribute('pluginpfad', $this->getPluginURL());
        $template->set_attribute('pfeil', ($this->hasVisited($a->getArtikelId()) ? "forumgrau" : "forumrot"));
        $template->set_attribute('pfeil_runter', "forumgraurunt");
        //benutzer und root extrafunktionen anzeigen
        if($a->getUserId() == $this->user->id || $this->perm->have_perm('root'))
        {
            $template->set_attribute('access', true);
            $template->set_attribute('link_delete', PluginEngine::getURL($this, array("artikel_id"=>$a->getArtikelId()), 'deleteArtikel'));
            $template->set_attribute('link_edit', PluginEngine::getURL($this, array("thema_id"=>$a->getThemaId(), "artikel_id"=>$a->getArtikelId()), 'editArtikel'));
        }
        // oder einen antwortbutton
        if($a->getUserId() != $this->user->id)
        {
            $template->set_attribute('antwort', true);
        }
        $template->set_attribute('link_search', PluginEngine::getURL($this, array("modus"=>"show_search_results")));
        $template->set_attribute('link_back', PluginEngine::getURL($this, array()));
        return $template->render();
    }

    /**
     * Holt die 20 (default) aktuellsten Artikel aus der Datenbank
     * Die Anzahl der Artikel wird in der globalen Konfiguration festgelegt
     *
     * @return array() Artikel
     */
    private function getLastArtikel()
    {
        $result = DBManager::get()->query("SELECT artikel_id FROM sb_artikel "
                      ."WHERE UNIX_TIMESTAMP() < (mkdate + {$this->zeit}) "
                      ."AND visible=1 ORDER BY mkdate DESC "
                      ."LIMIT {$this->announcements}")->fetchAll(PDO::FETCH_COLUMN);
        foreach ($result as $artikel_id) {
            $ret[] = new Artikel($artikel_id);
        }
        return $ret;
    }

	/**
	 *
	 */
	function ajaxDispatch_action()
	{
		$obj_id = Request::get('objid');
		$thema_id = Request::get('thema_id');
		//Artikel
		if ($obj_id){
			$oid = DBManager::get()->quote($obj_id );
			$uid = DBManager::get()->quote($GLOBALS['user']->id);
			DBManager::get()->exec("REPLACE INTO sb_visits SET object_id=$oid, user_id=$uid, type='artikel', last_visitdate=UNIX_TIMESTAMP()");
			$a = new Artikel($obj_id);
			Header('Content-Type: text/html; charset=windows-1252');
			echo $this->showArtikel($a, 'artikel_content');
            //nach dem verndern der themen, muss auch der cache geleert werden
            StudipCacheFactory::getCache()->expire(self::ARTIKEL_CACHE_KEY.$a->getThemaId());
		}
		//thema
		if($thema_id){
			$tt = $thema['thema'] = new Thema($thema_id);
			if($this->perm->have_perm($tt->getPerm(), $this->user->id) ||  $this->perm->have_perm('root'))
			{
				$thema['permission'] = true;
			}
			$thema['artikel'] = array();
			$artikel = $this->getArtikel($tt->getThemaId());
			foreach($artikel as $a)
			{
				array_push($thema['artikel'], $this->showArtikel($a));
			}
			$tt->setArtikelCount(count($artikel));
			$template = $this->template_factory->open('themen_artikel');
			$template->set_attribute('pluginpfad', $this->getPluginURL());
			$template->set_attribute('link_artikel', PluginEngine::getURL($this, array(), 'editArtikel'));
			$template->set_attribute('result', $thema);
			Header('Content-Type: text/html; charset=windows-1252');
			echo $template->render();
		}
	}

    /**
     * unschn aber erstmal duplikation der createQuestion mit anpassung fr plugins.
     *
     * @param $question
     * @param $approvalCmd
     */
    function createQuestion($question, $approvalCmd, $link = 'show')
    {
        $msg = $GLOBALS['template_factory']->open('shared/question');
        $msg->question = $question;
        $msg->approvalLink = PluginEngine::getURL($this, $approvalCmd, $link);
        $msg->disapprovalLink = PluginEngine::getURL($this);
        echo $msg->render();
    }
}
